use serde::{Deserialize, Serialize};

use crate::{Id, Inventory, Kind, Valid};

#[derive(Debug, PartialEq, Clone, Serialize, Deserialize, Default)]
pub struct Object {
    // Required Fields
    pub id: Id,
    pub kind: Kind,
    pub name: String,

    // Optional Fields
    pub password: String,
    pub owner: Id,
    pub location: Id,
    pub health: u64,
    pub max_health: u64,
    pub magic: u64,
    pub max_magic: u64,
    pub build: u64,
    pub max_build: u64,
    pub inv: Inventory
}

impl Valid for Object {
    fn valid(&self) -> bool {
        if self.kind == Kind::Player {
            if self.password.is_empty() {
                return false;
            }
        }
        self.id.valid() && self.kind.valid() && !self.name.is_empty()
    }
}

impl Object {
    pub fn set_password(&mut self, password: &String) -> bool {
        let hash = bcrypt::hash(password, bcrypt::DEFAULT_COST);
        match hash {
            Ok(hash) => {
                self.password = hash;
                true
            }
            Err(_) => false
        }
    }
    pub fn verify_password(&self, password: &String) -> bool {
        let verify = bcrypt::verify(password, self.password.as_str());
        match verify {
            Ok(ok) => ok,
            Err(_) => false
        }
    }

    pub fn load_file(filename: &String) -> Option<Object> {
        let p = crate::fs_util::join(&format!("{}", filename));
        let cont = crate::fs_util::read_file(&p);
        if cont.is_empty() {
            eprintln!("Err reading '{}'", p);
            return None;
        }
        let obj: Result<Object, serde_json::Error> = serde_json::from_str(cont.as_str());
        match obj {
            Ok(obj) => Some(obj),
            Err(_) => {
                eprintln!("Err parsing '{}'", p);
                None
            }
        }
    }
    pub fn save(&mut self) -> bool {
        if !self.valid() {
            eprintln!("Object not valid, won't save");
            return false;
        }
        let p = crate::fs_util::join(&format!("data/objects/{}.json", self.id));
        let dir = crate::fs_util::join(&"data/objects".to_string());
        if !crate::fs_util::dir_exists(&dir) {
            eprintln!("Missing data/objects dir, creating... '{}'", dir);
            if !crate::fs_util::mkdir(&dir) {
                eprintln!("Err while making data/objects dir, '{}'", dir);
            }
        }
        let cont = serde_json::to_string(self);
        match cont {
            Ok(cont) => {
                if !crate::fs_util::write_file(&p, &cont) {
                    eprintln!("Err while writing to '{}'", p);
                    return false;
                }
                true
            }
            Err(_) => {
                eprintln!("Err while serializing");
                false
            }
        }
    }
}
